import '../../fireflutter.dart';

class SendPushNotificationService {
  static SendPushNotificationService? _instance;
  static SendPushNotificationService get instance {
    _instance ??= SendPushNotificationService();
    return _instance!;
  }

  sendToAll(Map<String, dynamic> data) async {
    return FunctionsApi.instance
        .request('sendMessageToAll', data: data, addAuth: true);
  }

  sendToToken(Map<String, dynamic> data) async {
    return FunctionsApi.instance
        .request('sendMessageToTokens', data: data, addAuth: true);
  }

  sendToTopic(Map<String, dynamic> data) async {
    return FunctionsApi.instance
        .request('sendMessageToTopic', data: data, addAuth: true);
  }

  /// Send message to users.
  ///
  /// This is much the same as [sendToChatUser]
  ///
  /// [data] should have all the necessary properties to send it to backend.
  /// 'title' is the title of the message
  /// 'body' is the body of the message.
  /// 'uids' is a string of uid(s) separated by comma(,).
  /// 'badge' is the number of new chat message. It may not work on Android.
  /// 'type' is the type of message
  /// 'senderUid' is the sender uid.
  /// 'id' can be any id like postId
  sendToUsers(Map<String, dynamic> data) async {
    return FunctionsApi.instance
        .request('sendMessageToUsers', data: data, addAuth: true);
  }

  /// Sending message to the other user in chat room.
  ///
  /// Use this method to send push notification to the other user when login user sends a message to him.
  /// This push notification will lead to open chat room when it is tapped on system try. That's why it is for
  /// chat.
  ///
  /// [data] should have all the necessary properties to send push notifications to chat user.
  ///
  /// 'title' is the title of the message
  /// 'body' is the body of the message.
  /// 'uids' is a string of uid(s) separated by comma(,).
  /// 'badge' is the number of new chat message. It may not work on Android.
  /// 'type' is the type of message
  /// 'senderUid' is the sender uid.
  /// 'subscription' to check if the user has that subscription. If the user has no subscription, then message will not be sent.
  sendToChatUser(Map<String, dynamic> data) async {
    return FunctionsApi.instance
        .request('sendMessageToChatUser', data: data, addAuth: true);
  }
}
