import '../../../fireflutter.dart';

import 'package:flutter/material.dart';

class ChatRoomPushNotificationIcon extends StatefulWidget {
  ChatRoomPushNotificationIcon(
    this.uid, {
    this.size,
    this.hideOnDisabled = false,
  });
  final String uid;
  final double? size;
  final bool hideOnDisabled;
  @override
  _ChatRoomPushNotificationIconState createState() =>
      _ChatRoomPushNotificationIconState();
}

class _ChatRoomPushNotificationIconState
    extends State<ChatRoomPushNotificationIcon> {
  bool hasDisabledSubscription() {
    return UserService.instance.user.settings
        .hasDisabledSubscription('chatNotify' + widget.uid, 'chat');
  }

  bool loading = false;

  @override
  Widget build(BuildContext context) {
    if (widget.uid == '') return SizedBox.shrink();
    if (widget.hideOnDisabled && hasDisabledSubscription())
      return SizedBox.shrink();

    return GestureDetector(
      child: loading == false
          ? Icon(
              hasDisabledSubscription()
                  ? Icons.notifications_off
                  : Icons.notifications,
              color: Colors.grey.shade700,
              size: widget.size,
            )
          : SizedBox(
              width: widget.size,
              height: widget.size,
              child: CircularProgressIndicator.adaptive(),
            ),
      onTap: () => toggle(),
    );
  }

  toggle() async {
    if (UserService.instance.user.signedOut) {
      return showDialog(
        context: context,
        builder: (c) => AlertDialog(
          title: Text('notifications'),
          content: Text('login_first'),
        ),
      );
    }

    if (mounted)
      setState(() {
        loading = true;
      });
    await UserSettingService.instance.toggleTopic(
        'chatNotify' + widget.uid, 'chat', hasDisabledSubscription());
    if (mounted)
      setState(() {
        loading = false;
      });
    String msg = hasDisabledSubscription() ? 'unsubscribed' : 'subscribed';
    showDialog(
      context: context,
      builder: (c) => AlertDialog(
        title: Text('Notification'),
        content: Text(msg),
      ),
    );
  }
}
