// import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:flutter/material.dart';
import '../../../fireflutter.dart';

/// UserDoc
///
class UserDoc extends StatefulWidget {
  const UserDoc({
    required this.uid,
    required this.builder,
    this.reset = false,
    this.loader = const Center(
      child: SizedBox(
        width: 10,
        height: 10,
        child: CircularProgressIndicator.adaptive(
          strokeWidth: 2,
        ),
      ),
    ),
    Key? key,
  }) : super(key: key);
  final bool reset;
  final String uid;
  final Widget loader;
  final Widget Function(UserModel) builder;

  @override
  State<UserDoc> createState() => _UserDocState();
}

class _UserDocState extends State<UserDoc> {
  UserModel? user;

  @override
  void initState() {
    super.initState();
    init();
  }

  init() async {
    try {
      user = await UserService.instance
          .getOtherUserDoc(widget.uid, reset: widget.reset);

      if (mounted) setState(() => null);
    } on FirebaseException catch (e) {
      if (e.code == 'permission-denied') {
        debugPrint(
            '----> Error ${e.code} ${e.message} on "UserDoc::UserService.instance.getOtherUserDoc()". This permission-denied error is silently ignore since it is not important.');
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    if (user == null) return widget.loader;
    return widget.builder(user!);
  }
}
